/******************************************************************************\
**                                                                            **
** Licensed Materials - Property of IBM                                       **
** IBM Toolkit for MPEG-4                                                     **
** Copyright IBM Corp. 1998-2005  All Rights Reserved                         **
**                                                                            **
\******************************************************************************/

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.Rectangle;
import java.io.IOException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;                    // V3  See run method below
// import org.eclipse.swt.internal.awt.win32.SWT_AWT;  // V2  for more information
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import player.PlayerAdapter;
import player.PlayerControl;
import player.PlayerControlPanel;
import player.PlayerControlPanelFactory;
import player.PlayerFactory;
import player.PlayerListener;

/*******************************************************************************
 * IBM Toolkit for MPEG-4 Sample Player application for SWT. This code has been
 * adapted from the M4AppSample which was written using the AWT for the UI. The
 * code can thus be compared together to see the differences. The menus here are
 * in some cases MenuItems not Menus as they are under AWT. The naming has been
 * kept though the same as the M4AppSample.
 * <p>
 * This sample can be run under windows in Eclipse 2.0, using the internal win32
 * package; under Eclipse 3.0 the swt.awt package is used.
 *
 */
public class M4AppSampleSWT
{
  private final static String kPlayerName = "IBM Toolkit for MPEG-4 Sample Player";

  // The menus for the player; indented to show highlight the actual layout : STYLE
  // STYLE indicated for MenuItems if not PUSH
  //
  private MenuItem fileMenu;          // "File"
  private MenuItem openMenuItem;      //   "Open..."
  private MenuItem closeMenuItem;     //   "Close"
  private MenuItem exitMenuItem;      //   "Exit"

  private MenuItem viewMenu;          // "View"
  private MenuItem autosizeMenuItem;  //   "Size window to new content"        CHECK
  private MenuItem scaleMenuItem;     //   "Scale content to window"           CHECK
  private MenuItem speedScaleMenuItem;//   "Scaling; speed (vs. quality)"      CHECK
  private MenuItem sizeMenu;          //   "Window size"
  private MenuItem size50MenuItem;    //     "50%"
  private MenuItem size100MenuItem;   //     "100%"
  private MenuItem size200MenuItem;   //     "200%"
  private MenuItem sizeHalfMenuItem;  //     "x \u00BD"
  private MenuItem sizeTwiceMenuItem; //     "x 2"
  private MenuItem ctlBarMenuItem;    //   "Control Bar"                       CHECK

  private MenuItem playMenu;          // "Play"
  private MenuItem playPauseMenuItem; //   "Play/Pause"
  private MenuItem stopMenuItem;      //   "Stop"
  private MenuItem atDurActMenu;      //   "At duration"
  private MenuItem endContMenuItem;   //     "Continue"                        CHECK
  private MenuItem endStopMenuItem;   //     "Stop"                            CHECK
  private MenuItem endRptMenuItem;    //     "Restart"                         CHECK
  private MenuItem muteMenuItem;      //   "Mute"                              CHECK

  private Shell     shell;                     // SWT Shell
  private Composite composite;                 // Composite is parented by shell and contains SWT_AWT awt container
  private MyPanel   awtPanel;                  // Subclassed panel, added to above container, to which we add player

  private PlayerControl      m4Player;         // This is the control interface for the player
  private PlayerControlPanel m4PlayerPanel;    // This is for the control panel bar
  private PlayerListener     m4PlayerListener; // Our listener instance

  private boolean playerAdded;                 // True when player renderer component is added to our frame

  /*****************************************************************************
   * The constructor creates the player and sets up listeners, sets up
   * behavior using the player control, adds a player control panel
   * and shows the application
   */
  public M4AppSampleSWT()
  {
    // Create the MPEG-4 player and add our listener class that has been derived from the PlayerAdapter
    //
    m4Player = PlayerFactory.createMPEG4Player();
    m4PlayerListener = new M4AppPlayerAdapter();
    m4Player.addListener(m4PlayerListener);
  }

  /*****************************************************************************
   * Creates the window (shell) for the player and the SWT_AWT bridge which
   * allows us to use the player in the SWT UI. The method then has the SWT
   * event dispatch loop which runs until the application exits.
   */
  private final void run(String[] args)
  {
    final Display display = new Display();
    shell    = new Shell(display);

    Container container;

    // Version 3 versus Version 2.
    //
    // Following logic is for V3 and V2 versions of SWT (Eclipse). V2 uses internal win32 package. Comment and
    // uncomment V3 and V2 lines accordingly below. This is just the following pairs of lines and also one
    // instance in the imports above.

    composite = new Composite(shell, SWT.EMBEDDED);  // V3  Note: SWT.EMBEDDED type so as can contain AWT Frame
    container = SWT_AWT.new_Frame(composite);        // V3        from SWT_AWT package

//  composite = new Composite(shell, SWT.NONE);      // V2  Use this pair of lines, instead of the above,
//  container = SWT_AWT.new_Panel(shell);            // V2  to run using Version 2.

    container.setLayout(new BorderLayout());
    awtPanel = new MyPanel();
    awtPanel.setLayout(new BorderLayout());
    container.add("Center", awtPanel);

    createContents();
    composite.setBounds(shell.getClientArea());

    // This is to close and exit our player properly, eg stop playing content, when we click the close button
    //
    shell.addShellListener(new ShellAdapter() {
      public void shellClosed(ShellEvent event)
      {
        exit();
      }
    });

    shell.addControlListener(new ControlAdapter() {
      public void controlResized(ControlEvent event)
      {
        composite.setBounds(shell.getClientArea());
      }
    });

    shell.open();

    if (args.length > 0)
    {
      openPlayUrl(args[0]);  // invoke content directly from the command line
    }

    while(!shell.isDisposed())
    {
      if (!display.readAndDispatch()) display.sleep();
    }
    display.dispose();

    // Just existing the usual SWT dispatch loop does not allow the player to exit when invoked
    // in Windows from the command line after it been running audio. JavaSound creates some
    // extra threads which perhaps cause an issue. Anyway adding this System.exit call does it.
    // Note: when the player is launched from in Eclipse IDE then it did end ok without this...
    //
    System.exit(0);
  }

  /*****************************************************************************
   * Creates the shell contents by adding a menu bar player and a player
   * control panel. Shell title and size is set too. The player will be added
   * later when content is opened - for now the text message from MyPanel class
   * will show.
   */
  private final void createContents()
  {
    // Create the menus bar and setup the player
    //
    createMenuBar();
    //
    m4Player.setAutoSize(false);                       // We don't want the forced autosize, we do any resize ourself
    m4Player.setScaling(scaleMenuItem.getSelection()); // Scaling is set as per the menu state
    m4Player.setSpeedScaling(speedScaleMenuItem.getSelection());
    setEndAction(PlayerControl.END_ACTION_CONTINUE);   // We set a default end behavior to carry on playing
    atDurActMenu.setEnabled(false);                    // wait until we have content to enable this (see M4AppPlayerAdapter)
    m4Player.setMute(muteMenuItem.getSelection());     // Mute is as per the menu state

    // Create a control panel and set the menus to match
    //
    m4PlayerPanel = PlayerControlPanelFactory.createPlayerControlPanel("Basic", m4Player);
    if (m4PlayerPanel != null)
    {
      m4PlayerPanel.setBackground(java.awt.Color.lightGray);
      showControlBar(true);
      ctlBarMenuItem.setEnabled(true);
      ctlBarMenuItem.setSelection(true);
    }

    // Lastly put the sample into the window title and show the player at a reasonable starting size
    //
    shell.setText(kPlayerName);
    shell.setSize(360, 280);
  }


  /*****************************************************************************
   * Creates the menus bar, sets up listeners for the menus and initializes
   * menus according to an initial setup (Note: some menu settings could be
   * saved as persistent preferences and reloaded, but that is beyond the scope
   * of this sample player)
   */
  private final void createMenuBar()
  {
    Menu menu = new Menu(shell, SWT.BAR);

    // FILE ------------------------------------------------------------------------
    //
    fileMenu = new MenuItem(menu, SWT.CASCADE);
    fileMenu.setText("File");
    Menu dropMenu = new Menu(shell, SWT.DROP_DOWN);
    fileMenu.setMenu(dropMenu);
    //
    openMenuItem = new MenuItem(dropMenu, SWT.PUSH);
    openMenuItem.setText("Open...\tCtrl+O");
    openMenuItem.setAccelerator(SWT.CTRL + 'O');
    openMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        open();
      }
    });
    //
    closeMenuItem = new MenuItem(dropMenu, SWT.PUSH);
    closeMenuItem.setText("Close");
    closeMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        close();
      }
    });
    //
    MenuItem item = new MenuItem(dropMenu, SWT.SEPARATOR);
    //
    exitMenuItem = new MenuItem(dropMenu, SWT.PUSH);
    exitMenuItem.setText("Exit");
    exitMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        exit();
      }
    });

    // VIEW ------------------------------------------------------------------------
    //
    viewMenu = new MenuItem(menu, SWT.CASCADE);
    viewMenu.setText("View");
    dropMenu = new Menu(shell, SWT.DROP_DOWN);
    viewMenu.setMenu(dropMenu);
    //
    autosizeMenuItem = new MenuItem(dropMenu, SWT.CHECK);
    autosizeMenuItem.setText("Size window to new content");
    //
    scaleMenuItem = new MenuItem(dropMenu, SWT.CHECK);
    scaleMenuItem.setText("Scale content to window");
    scaleMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        m4Player.setScaling(scaleMenuItem.getSelection());
      }
    });
    //
    speedScaleMenuItem = new MenuItem(dropMenu, SWT.CHECK);
    speedScaleMenuItem.setText("Scaling; speed (vs. quality)");
    speedScaleMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        m4Player.setSpeedScaling(speedScaleMenuItem.getSelection());
      }
    });
    //
    sizeMenu = new MenuItem(dropMenu, SWT.CASCADE);
    sizeMenu.setText("Window Size");
    Menu sizeDropMenu = new Menu(shell, SWT.DROP_DOWN);
    sizeMenu.setMenu(sizeDropMenu);
    //
    size50MenuItem = new MenuItem(sizeDropMenu, SWT.PUSH);
    size50MenuItem.setText("50%\tCtrl+0");
    size50MenuItem.setAccelerator(SWT.CTRL + '0');
    size50MenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setSizePercent(50);
      }
    });
    //
    size100MenuItem = new MenuItem(sizeDropMenu, SWT.PUSH);
    size100MenuItem.setText("100%\tCtrl+1");
    size100MenuItem.setAccelerator(SWT.CTRL + '1');
    size100MenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setSizePercent(100);
      }
    });
    //
    size200MenuItem = new MenuItem(sizeDropMenu, SWT.PUSH);
    size200MenuItem.setText("200%\tCtrl+2");
    size200MenuItem.setAccelerator(SWT.CTRL + '2');
    size200MenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setSizePercent(200);
      }
    });
    //
    item = new MenuItem(sizeDropMenu, SWT.SEPARATOR);
    //
    sizeHalfMenuItem = new MenuItem(sizeDropMenu, SWT.PUSH);
    sizeHalfMenuItem.setText("x \u00BD\tCtrl+F1");
    sizeHalfMenuItem.setAccelerator(SWT.CTRL + SWT.F1);
    sizeHalfMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setSizeActual(50);
      }
    });
    //
    sizeTwiceMenuItem = new MenuItem(sizeDropMenu, SWT.PUSH);
    sizeTwiceMenuItem.setText("x 2\tCtrl+F2");
    sizeTwiceMenuItem.setAccelerator(SWT.CTRL + SWT.F2);
    sizeTwiceMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setSizeActual(200);
      }
    });
    //
    item = new MenuItem(dropMenu, SWT.SEPARATOR);
    //
    ctlBarMenuItem = new MenuItem(dropMenu, SWT.CHECK);
    ctlBarMenuItem.setText("Control Bar");
    ctlBarMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        showControlBar(ctlBarMenuItem.getSelection());
      }
    });

    // PLAY ------------------------------------------------------------------------
    //
    playMenu = new MenuItem(menu, SWT.CASCADE);
    playMenu.setText("Play");
    dropMenu = new Menu(shell, SWT.DROP_DOWN);
    playMenu.setMenu(dropMenu);
    //
    playPauseMenuItem = new MenuItem(dropMenu, SWT.PUSH);
    playPauseMenuItem.setText("Play/Pause\tCtrl+P");
    playPauseMenuItem.setAccelerator(SWT.CTRL + 'P');
    playPauseMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        playPauseAction();
      }
    });
    //
    stopMenuItem = new MenuItem(dropMenu, SWT.PUSH);
    stopMenuItem.setText("Stop\tCtrl+S");
    stopMenuItem.setAccelerator(SWT.CTRL + 'S');
    stopMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        stopAction();
      }
    });
    //
    item = new MenuItem(dropMenu, SWT.SEPARATOR);
    //
    //
    atDurActMenu = new MenuItem(dropMenu, SWT.CASCADE);
    atDurActMenu.setText("At duration");
    Menu atDurActDropMenu = new Menu(shell, SWT.DROP_DOWN);
    atDurActMenu.setMenu(atDurActDropMenu);
    //
    endContMenuItem = new MenuItem(atDurActDropMenu, SWT.CHECK);
    endContMenuItem.setText("Continue");
    endContMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setEndAction(PlayerControl.END_ACTION_CONTINUE);
      }
    });
    //
    endStopMenuItem = new MenuItem(atDurActDropMenu, SWT.CHECK);
    endStopMenuItem.setText("Stop");
    endStopMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setEndAction(PlayerControl.END_ACTION_STOP);
      }
    });
    //
    endRptMenuItem = new MenuItem(atDurActDropMenu, SWT.CHECK);
    endRptMenuItem.setText("Restart");
    endRptMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        setEndAction(PlayerControl.END_ACTION_REPEAT);
      }
    });
    //
    item = new MenuItem(dropMenu, SWT.SEPARATOR);
    //
    muteMenuItem = new MenuItem(dropMenu, SWT.CHECK);
    muteMenuItem.setText("Mute\tCtrl+M");
    muteMenuItem.setAccelerator(SWT.CTRL + 'M');
    muteMenuItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        m4Player.setMute(muteMenuItem.getSelection());
      }
    });

    closeMenuItem.setEnabled(false);
    autosizeMenuItem.setSelection(true);
    sizeMenu.setEnabled(false);
    ctlBarMenuItem.setEnabled(false);
    playPauseMenuItem.setEnabled(false);
    stopMenuItem.setEnabled(false);

    shell.setMenuBar(menu);
  }

  /*****************************************************************************
   * Main entry point for the sample application.  We can pass a url from the
   * command line here; a local file or remote server e.g.  rtsp:// type url.
   * With this sample application passing a remote url here is the only way to
   * access remote content as this sample player has only a file open dialog.
   * (An OpenURL dialog could be added but again is beyond the scope of this
   * sample).
   *
   * @param args A string as a url point to media content
   */
  public static void main(String[] args)
  {
    // Create a new instance of our application and run it
    new M4AppSampleSWT().run(args);
  }

  /*****************************************************************************
   * This opens and plays the content referenced by the url. The
   * player renderer component is added if it has not already been added.
   * (i.e. we can Open a new content without closing the prior one).
   *
   * @param url    String referencing the content e.g. myfile.mp4,
   *               rtsp://myserver.com/myfile.mp4
   */
  private void openPlayUrl(String url)
  {
    // Stop & close any playback that may be running still
    //
    m4Player.stopUrl();

    try
    {
      // We title our window as per the content and added the player renderer
      // component if needed. Now we have the player added we can enable the size
      // menu to allow the content display size to be set.
      //
      shell.setText(url);
      if (!playerAdded)
      {
        awtPanel.add("Center", m4Player.getRendererComponent());
        playerAdded = true;
        sizeMenu.setEnabled(true);
        awtPanel.validate();
      }

      m4Player.open(url);
      m4Player.start();
    }
    catch (Exception e)
    {
      m4Player.stopUrl();                  // Stop MPEG-4 player
      shell.setText(e.toString());         // and set error in title bar
    }
  }

  /*****************************************************************************
   * Opens a local file from the file dialog and plays it
   */
  private void open()
  {
    FileDialog openFileDialog = new FileDialog(shell);
    String fName= openFileDialog.open();
    if (fName != null)
    {
      openPlayUrl(fName);
    }
  }

  /*****************************************************************************
   * Stop playing and close any content, remove the player renderer
   * component. By closing the content we close any connection to it, e.g.. over
   * RTSP stopURL() will teardown and end the session.
   */
  private void close()
  {
    m4Player.stopUrl();
    awtPanel.remove(m4Player.getRendererComponent());
    sizeMenu.setEnabled(false);
    playerAdded = false;
    shell.setText(kPlayerName);
  }

  /*****************************************************************************
   * We want to exit the application. Stop and close any content
   * that may be playing before exiting.
   */
  private void exit()
  {
    m4Player.stopUrl();
    m4Player.removeListener(m4PlayerListener);
    shell.dispose();
  }

  /*****************************************************************************
   * This sets the player size as a ratio of the preferred content size. The
   * menus are not enabled unless there is a preferred size. An
   * optional preferred size is signalled in MPEG-4 content.
   *
   * @param percent Percentage of preferred size. Currently 50%, 100% and 200%
   */
  private void setSizePercent(int percent)
  {
    int wTarget = m4Player.getPreferredSceneSize().width;
    int hTarget = m4Player.getPreferredSceneSize().height;

    switch (percent)
    {
      case  50: wTarget /= 2; hTarget /= 2; break;
      case 100:                             break;
      case 200: wTarget *= 2; hTarget *= 2; break;
      default:  throw new IllegalArgumentException();
    }

    setSize(wTarget, hTarget);
  }

  /*****************************************************************************
   * This sets the player size as a ratio of its current size
   * whatever that is.
   *
   * @param percent Percentage of current size to set as new size (50% or 200%)
   */
  private void setSizeActual(int percent)
  {
    int wTarget = m4Player.getSize().width;
    int hTarget = m4Player.getSize().height;

    switch (percent)
    {
      case  50: wTarget /= 2; hTarget /= 2; break;
      case 200: wTarget *= 2; hTarget *= 2; break;
      default:  throw new IllegalArgumentException();
    }

    setSize(wTarget, hTarget);
  }

  /*****************************************************************************
   * Sets the size for the content being played by setting the size into the
   * player and also adjusting the shell size. The shell size must include space
   * for the control panel when it is visible.
   *
   * @param w      Required player width
   * @param h      Required player height
   */
  private void setSize(final int w, final int h)
  {
    m4Player.setSize(w, h);
    awtPanel.validate();

    composite.setSize(w, h + (ctlBarMenuItem.getSelection() ? m4PlayerPanel.getHeight() : 0));
    shell.pack();
  }

  /*****************************************************************************
   * This allows the control bar panel to be hidden or shown. If there
   * is no player renderer component added then the panel is added. If
   * the player renderer component has been added then the layout is
   * adjusted to keep the player size and add the panel.
   *
   * @param show   Set to true to show (add) the panel. False to hide (remove).
   */
  private void showControlBar(boolean show)
  {
    if (show)
    {
      awtPanel.add("South", m4PlayerPanel);
      m4PlayerPanel.attachToPlayer();
    }
    else
    {
      awtPanel.remove(m4PlayerPanel);
      m4PlayerPanel.detachFromPlayer();
    }
    if (playerAdded)
    {
      setSize(m4Player.getSize().width, m4Player.getSize().height);
    }
    else
    {
      if (show)
      {                               // Hack....
        Point p = shell.getSize();    // When player is closed (e.g as it starts)
        p.x +=1;                      // the Control Bar does not paint properly
        shell.setSize(p);             // when added back. However manually resizing
        p.x -=1;                      // did make it repaint ok, this simulates that
        shell.setSize(p);             // Not pretty but it seems to do the job.
      }
    }
  }

  /*****************************************************************************
   * Handle our pause/play menu by playing (start()), or pausing or resuming
   * dependent on state. Menus are set as per state callbacks on our listener
   * class that extends PlayerAdapter.
   *
   * Error handling could be better; IOExceptions may occur for example if we
   * are using RTSP to communicate with a server over a network.
   */
  void playPauseAction()
  {
    try
    {
      switch (m4Player.getState())
      {
        case PlayerControl.STOPPED: m4Player.start();  break;
        case PlayerControl.PAUSED:  m4Player.resume(); break;
        case PlayerControl.PLAYING: m4Player.pause();  break;
      }
    }
    catch (IOException e){/*nothing*/}
    catch (IllegalStateException ie){/*nothing*/}
  }

  /*****************************************************************************
   * Handle our stop menu by stopping the player
   */
  void stopAction()
  {
    try
    {
      m4Player.stop();
    }
    catch (IOException e){/*nothing*/}
    catch (IllegalStateException ie){/*nothing*/}
  }

  /*****************************************************************************
   * Set the action that the player will take when the indicated duration of
   * the content is reached (if the content duration is indefinite/unknown
   * then this has no net effect)
   *
   * @param endAction The action to take
   */
  private void setEndAction(int endAction)
  {
    endContMenuItem.setSelection(endAction == PlayerControl.END_ACTION_CONTINUE);
    endStopMenuItem.setSelection(endAction == PlayerControl.END_ACTION_STOP);
    endRptMenuItem.setSelection(endAction  == PlayerControl.END_ACTION_REPEAT);

    m4Player.setPlayerEndAction(endAction);
  }


  /*****************************************************************************
   * Extends the PlayerAdapter class so we can implement the methods only for
   * those events we are interested in here.
   * <p>
   * Since the methods here do UI actions we must ensure they are done on the
   * UI thread. To avoid any possibility of deadlock with the player, if/when
   * any methods need to call back to the player, we use the asyncExec() rather
   * than the syncExec().
   *
   * @see player.PlayerAdapter
   * @see player.PlayerListener
   */
  private class M4AppPlayerAdapter extends PlayerAdapter
  {
    private boolean autoSized;  // Used when autosizing from content to content so we can prevent
                                // resizing when stopping/playing same content

    /***************************************************************************
     * Enable close/play/pause/stop menus according to state.  This allows us to
     * work in conjunction with the player control panel that can also affect
     * state.
     *
     * @param state  State as per PlayerControl i.e.. Closed. Playing etc
     *
     * @see player.PlayerListener#changedState(int)
     */
    public void changedState(final int state)
    {
      shell.getDisplay().asyncExec(new Runnable() {
        public void run()
        {
          if (!shell.isDisposed())  // Make sure when we exit that we don't
          {                         // cause any exception by processing state
            switch (state)          // changes after shell is disposed
            {
              case PlayerControl.CLOSED:
                closeMenuItem.setEnabled(false);
                playPauseMenuItem.setEnabled(false);
                break;

              case PlayerControl.STOPPED:
                closeMenuItem.setEnabled(true);
                playPauseMenuItem.setEnabled(true);
                stopMenuItem.setEnabled(false);

                // The player is in Stopped state after the content is opened. In this
                // state we now have access to content specific info such as duration,
                // whether we can seek etc. Here this is used to show that we can
                // access if the content has a duration and enable/disable the
                // endAction menu accordingly (Note: since this is more a player
                // preference type behavior it may be preferable to keep the menu
                // enabled, whether specific content has duration or not, as it done
                // in our M4Play application. But this is shown here to illustrate
                // that certain info is available once the content is opened. See the
                // PlayerControl javadoc for more info.)
                //
                //
                atDurActMenu.setEnabled(m4Player.hasDuration());
                break;

              case PlayerControl.PLAYING:
                stopMenuItem.setEnabled(true);
                break;

              case PlayerControl.PAUSED:
                break;
            }
          }
        }
      });
    }

    /***************************************************************************
     * Reflect mute state on our menu
     *
     * @param isMuted True when audio playback is muted
     *
     * @see player.PlayerListener#muteChanged(boolean)
     */
    public void muteChanged(final boolean isMuted)
    {
      shell.getDisplay().asyncExec(new Runnable() {
        public void run()
        {
          muteMenuItem.setSelection(isMuted);
        }
      });
    }

    /***************************************************************************
     * This is called to signal optional preferred size from the content.  If we
     * have a preferred size we can enable the menus that allow the size to be
     * set as a ratio of preferred size.  Also, if we autosize to new content we
     * will adjust the player size accordingly.  We use the autoSized flag as we
     * get signalled for this preferred size each time if/when we stop and play
     * the content and for our application we will leave the size at whatever
     * the user wants as long as we don't change to another piece of content.
     * In the latter case we change size only then if the menu preference is
     * selected.
     *
     * @param w      Preferred width or -1 if no preferred size
     * @param h      Preferred height or -1 if no preferred size
     *
     * @see player.PlayerListener#preferSize(int, int)
     */
    public void preferSize(final int w, final int h)
    {
      shell.getDisplay().asyncExec(new Runnable() {
        public void run()
        {
          boolean hasPreferredSize = (w > 0 && h > 0);

          size50MenuItem.setEnabled(hasPreferredSize);
          size100MenuItem.setEnabled(hasPreferredSize);
          size200MenuItem.setEnabled(hasPreferredSize);

          if (autosizeMenuItem.getSelection() && hasPreferredSize)
          {
            if (!autoSized)
            {
              autoSized  = true;
              setSize(w, h);
            }
          }

          // Speed scaling is only when we have a preferred size
          //
          speedScaleMenuItem.setEnabled(hasPreferredSize);
        }
      });
    }

    /*****************************************************************************
     * Indicates pixel or meter metrics from presentation
     *
     * @param usePixelMetrics True if using pixel metrics, false is meter metrics
     */
    public void usingMetrics(final boolean usePixelMetrics)
    {
      // Scaling is always on in meter metrics so only enable the menu in pixel
      // metrics. This is a choice we could leave the menu always enabled.
      //
      shell.getDisplay().asyncExec(new Runnable() {
        public void run()
        {
          scaleMenuItem.setEnabled(usePixelMetrics);
        }
      });
    }

    /***************************************************************************
     * Content has been opened. Allow any auto resizing to occur and
     * set the window title to reflect the content. This is the first indication
     * we get following content being opened and precedes the preferredSize and
     * usingMetrics indications. Hence this is a suitable point to reset the
     * autosizing flag since the content has changed (it may be the same url if
     * the content is closed and re-opened but we handle that as if it was new
     * content and enable the autosizing again).
     *
     * @param url    The string for the content url.
     *
     * @see player.PlayerListener#urlChanged(java.lang.String)
     */
    public void urlChanged(final String url)
    {
      shell.getDisplay().asyncExec(new Runnable() {
        public void run()
        {
          autoSized = false;
          shell.setText(url);
        }
      });
    }
  }


  /*****************************************************************************
   * Extends the standard Panel class. This is just a convenience to allow the
   * painting of the sample 'logo'. The sample would work just as well with the
   * standard Panel.
   *
   */
  private class MyPanel extends Panel
  {
    /****************************************************************************
     * This method is just to display the sample player string in the main frame
     * before we add the player renderer component, i.e.  before we Open any
     * content.  or after the component is removed which is after when we Close
     * the content (i.e when the player is not added to the frame)
     *
     * @param g      Graphics context
     */
    public void paint(Graphics g)
    {
      if (!playerAdded)
      {
        Rectangle r = getBounds();
        int w = r.width;
        int h = r.height;

        g.setColor(Color.black);
        g.fillRect(0, 0, w, h);

        Insets i = getInsets();
        w -= i.left + i.right;
        h -= i.top  + i.bottom;

        g.setFont(logoFont);
        FontMetrics fm = g.getFontMetrics();
        int sWidth = fm.stringWidth(kPlayerName);
        int iLeft  = i.left + (w - sWidth)/2;
        int y      = i.top  + h/2;
        g.setColor(Color.blue);
        g.drawString(kPlayerName, iLeft, y);
      }
    }
    //
    private Font logoFont = new Font("Helvetica", Font.BOLD, 16);
  }

}
